/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.toFilenameTested(__filename),
    stj = require(env.lib('stj')),
    iface = require(__filenameTested);

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([__filenameTested, __filename]);

/*******************************************************************************
 * InterfaceTest class
 ******************************************************************************/
exports.InterfaceTest = vows.describe('Interface module').addBatch({

    "interface_()": {
        topic: function () {
            return stj.interface_;
        },
        "should return a new Interface": function (topic) {
            var Interface = stj.use_("Interface");

            assert.equal(topic("", { foo: true}) instanceof Interface, true);
        },
        "should return accept only true or Functions as parameters": function (topic) {
            assert.throws(function () {
                topic("", { foo: "fdsfsd"});
            });
            assert.throws(function () {
                topic("", { foo: 1});
            });
        },
        "should validate another object": function (topic) {
            var iface = topic("", {
                foo: true,
                bar: function (arg1, arg2) {},
                baz: function (arg1) {}
            });


            assert.throws(function () {
                iface.validate({
                    foo: function () {},
                    bar: function (arg1, arg2) {}
                });
            });

            assert.throws(function () {
                iface.validate({
                    foo: function () {},
                    bar: function (arg1, arg2) {},
                    baz: "string"
                });
            });

            assert.throws(function () {
                iface.validate({
                    foo: function () {},
                    bar: function (arg1, arg2) {},
                    baz: function () {}
                });
            });

            assert.doesNotThrow(function () {
                iface.validate({
                    foo: function () {},
                    bar: function (arg1, arg2) {},
                    baz: function (arg1, arg2) {}
                });
            });
        },
        "should throw error with corresponding message": function (topic) {
            var iface = topic("test.foo.ITest", {
                foo: true,
                bar: function (arg1, arg2) {},
                baz: function (arg1) {}
            });


            try {
                iface.validate({
                    foo: function () {},
                    bar: function (arg1, arg2) {}
                });
            } catch (e1) {
                assert.equal(e1.message, 'test.foo.ITest#baz() is not implemented.');
            }

            try {
                iface.validate({
                    foo: function () {},
                    bar: function (arg1, arg2) {},
                    baz: "string"
                });
            } catch (e2) {
                assert.equal(e2.message, 'test.foo.ITest#baz() is not implemented.');
            }
        }
    }
});