/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.toFilenameTested(__filename),
    Url = require(__filenameTested);

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([__filenameTested, __filename]);


/*******************************************************************************
 * UrlTest
 ******************************************************************************/
function createUrl(options) {
    return new Url(options);
}

exports.UrlTest = vows.describe('Url class').addBatch({
    'new Url({...})': {
        topic: function () {
            return createUrl({
                protocol: 'http',
                auth: null,
                hostname: 'localhost',
                port: 333,
                path: ['', 'toto', 'tata'],
                hash: 'isch',
                query: {
                    foo: 'bar'
                },
                nonexistent: 'foo'
            });
        },
        'should return an instance of Url': function (topic) {
            assert.ok(Object.isInstanceOf(topic, Url));
        },
        'should create an Url with corresponding attributes ': function (topic) {
            assert.deepEqual(topic, {
                protocol: 'http',
                auth: null,
                hostname: 'localhost',
                port: 333,
                path: ['', 'toto', 'tata'],
                hash: 'isch',
                query: {
                    foo: 'bar'
                }
            });
            assert.equal(topic.get('host'), 'localhost:333');
            assert.equal(topic.get('pathname'), '/toto/tata');
            assert.equal(topic.get('search'), '?foo=bar');
        },
        'should have valid string representation': function (topic) {
            assert.deepEqual(topic.toString(), 'http://localhost:333/toto/tata?foo=bar#isch');
        }
    },
    'new Url("...")': {
        topic: function () {
            return createUrl('http://localhost:333/toto/tata?foo=bar#isch');
        },
        'should return an instance of Url': function (topic) {
            assert.ok(Object.isInstanceOf(topic, Url));
        },
        'should create an Url with corresponding attributes ': function (topic) {
            assert.deepEqual(topic, {
                protocol: 'http',
                auth: null,
                hostname: 'localhost',
                port: 333,
                path: ['', 'toto', 'tata'],
                hash: 'isch',
                query: {
                    foo: 'bar'
                }
            });
            assert.equal(topic.get('host'), 'localhost:333');
            assert.equal(topic.get('pathname'), '/toto/tata');
            assert.equal(topic.get('search'), '?foo=bar');
        },
        'should have valid string representation': function (topic) {
            assert.deepEqual(topic.toString(), 'http://localhost:333/toto/tata?foo=bar#isch');
        }
    },
    'new Url(new Url(...))': {
        topic: function () {
            return createUrl(
            createUrl({
                protocol: 'http',
                auth: null,
                hostname: 'localhost',
                port: 333,
                path: ['', 'toto', 'tata'],
                hash: 'isch',
                query: {
                    foo: 'bar'
                },
                nonexistent: 'foo'
            }));
        },
        'should return an instance of Url': function (topic) {
            assert.ok(Object.isInstanceOf(topic, Url));
        },
        'should create an Url with corresponding attributes ': function (topic) {
            assert.deepEqual(topic, {
                protocol: 'http',
                auth: null,
                hostname: 'localhost',
                port: 333,
                path: ['', 'toto', 'tata'],
                hash: 'isch',
                query: {
                    foo: 'bar'
                }
            });
            assert.equal(topic.get('host'), 'localhost:333');
            assert.equal(topic.get('pathname'), '/toto/tata');
            assert.equal(topic.get('search'), '?foo=bar');
        },
        'should have valid string representation': function (topic) {
            assert.deepEqual(topic.toString(), 'http://localhost:333/toto/tata?foo=bar#isch');
        }
    }
});