/*jslint nodejs: true, indent:4 */
var path = require('path'),
    fs = require('fs'),
    vows = require('vows');

var env = {},
    port = 55000;
function searchRoot(root) {
    if (env.ROOT || env.length === 0) {
        return;
    }
    try {
        fs.statSync(path.join(root, 'package.json'));

        env.ROOT = root;
        env.LIB = env.LIB || path.join(env.ROOT, '');
        env.TEST = env.__test || path.join(env.ROOT, 'test');
        env.RESOURCE = env.__resource || path.join(env.ROOT, 'resource');

        env.TEST_SUFFIX = '-test.js';
        /*if (require.paths.indexOf(env.LIB)) {
            require.paths.push(env.LIB);
        }*/
    } catch (e) {
        searchRoot(path.dirname(root));
    }
}
searchRoot(__dirname);

//module.exports = env;
/*for (var property in env) {
    if (env.hasOwnProperty(property)) {
        global[property] = env[property];
    }
}*/

env.getNewPort = function () {
    var freePort = port;
    port += 1;
    return freePort;
};

env.lib = function (filename) {
    return path.join(env.LIB, filename);
};

env.toFilenameTested = function (filename) {
    var root = path.dirname(filename).replace(env.TEST, env.LIB),
        pathnames = [
            path.join(
                root,
                path.basename(filename).replace(env.TEST_SUFFIX, '.js')
            ),
            path.join(
                root,
                path.basename(filename).replace(env.TEST_SUFFIX, ''),
                'index.js'
            )
        ],
        pathname, i;
    for (i = 0; i < pathnames.length; i += 1) {
        pathname = pathnames[i];

        try {
            fs.lstatSync(pathname);
        } catch (e) {
            continue;
        }
        return pathname;
    }

    throw new Error('cannot find module tested for "' + filename + '"');
};

env.JSLintTest = function (filenames) {
    var lint;
    try {
        lint = require('lint');
    } catch (e) {
        console.warn('Warning: JSLint not found try `npm install lint`');
    }

    if (lint) {
        return lint.vows.createTest(filenames);
    }
};

//define shim
if (!global.define) {
    var currentModule,
        defaultCompile = module.constructor.prototype._compile,
        defaultInjects = ["require", "exports", "module"];
    module.constructor.prototype._compile = function _compile(content, filename) {
        currentModule = this;
        try {
            return defaultCompile.call(this, content, filename);
        } finally {
            currentModule = null;
        }
    };

    global.define = function define(id, injects, factory) {
        if (currentModule === null) {
            throw new Error("define() may only be called during module factory instantiation");
        }
        var module = currentModule,
            req = function (relativeId) {
                var lastId;

                if (relativeId.charAt(0) === '.') {
                    relativeId = id.substring(0, id.lastIndexOf('/') + 1) + relativeId;
                    while (lastId !== relativeId) {
                        lastId = relativeId;
                        /*jslint regexp: false*/
                        relativeId = relativeId.replace(/\/[^\/]*\/\.\.\//, '/');
                        /*jslint regexp: true*/
                    }
                    relativeId = relativeId.replace(/\/\.\//g, '/');
                }
                return require(relativeId);
            };
        if (!factory) {
            // two or less arguments
            factory = injects;
            if (factory) {
                // two args
                if (typeof id === "string") {
                    if (id !== module.id) {
                        throw new Error("Can not assign module to a different id than the current file");
                    }
                    // default injects
                    injects = defaultInjects;
                } else {
                    // anonymous, deps included
                    injects = id;
                }
            } else {
                // only one arg, just the factory
                factory = id;
                injects = defaultInjects;
            }
        }
        module.id = id;
        if (typeof factory !== "function") {
            module.exports = factory;
            // we can just provide a plain object
            return factory;
        }
        module.exports = factory.apply(module.exports, injects.map(function (injection) {
            switch (injection) {
            // check for CommonJS injection variables
            case "require":
                return req;
            case "exports":
                return module.exports;
            case "module":
                return module;
            default:
                // a module dependency
                return req(injection);
            }
        }));
    };
}

module.exports = env;