/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.toFilenameTested(__filename),
    inflect = require(__filenameTested);

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([__filenameTested, __filename]);

/*******************************************************************************
 * InflectTest
 ******************************************************************************/
exports.InflectTest = vows.describe('inflect module').addBatch({
    'inflect.camelize': {
        topic: function () {
            return inflect.camelize;
        },
        "should have length of 1": function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return empty string for an empty string': function (topic) {
            assert.equal(topic(''), '');
        },
        'should return empty string for a string containing only separators': function (topic) {
            assert.equal(topic('-_-___-'), '');
        },
        'should return a camelized string': function (topic) {
            assert.equal(topic('foo-bar-baz'), 'fooBarBaz');
            assert.equal(topic('foo_bar_baz'), 'fooBarBaz');
            assert.equal(topic('-foo-bar-baz'), 'FooBarBaz');
            assert.equal(topic('_foo_bar_baz'), 'FooBarBaz');
        },
        'should return a camelized string if many separators are present': function (topic) {
            assert.equal(topic('foo--bar-_baz'), 'fooBarBaz');
        }
    }
});