/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.lib('stj'),
    nativeMethods = {
        toString: Number.prototype.toString
    },
    stj = require(__filenameTested),
    object = stj.use_('Object'),
    number = stj.use_('Number');

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([/*__filenameTested, */__filename]);

/*******************************************************************************
 * NumberTest class
 ******************************************************************************/
exports.NumberTest = vows.describe('String functions').addBatch({
    "[native methods]": {
        topic: function () {
            return number.prototype;
        },
        'should not change native implementation': function (topic) {
            for (var methodName in nativeMethods) {
                if (nativeMethods.hasOwnProperty(methodName)) {
                    assert.equal(nativeMethods[methodName], topic[methodName]);
                    assert.equal(topic[methodName].toString(), "function " + methodName + "() { [native code] }");
                }
            }
        }
    },
    "isOdd()": {
        topic: function () {
            return number.prototype;
        },
        'should return true only for odd numbers (-1, 1, 3, 5, ...)': function (topic) {
            assert.equal((-1).isOdd(), true);
            assert.equal((1).isOdd(), true);
            assert.equal((3).isOdd(), true);
            assert.equal((4).isOdd(), false);
            assert.equal((2).isOdd(), false);
        }
    },
    "isEven()": {
        topic: function () {
            return number.prototype;
        },
        'should return true only for odd numbers (-2, 0, 2, 4, 6, ...)': function (topic) {
            assert.equal((-1).isEven(), false);
            assert.equal((1).isEven(), false);
            assert.equal((3).isEven(), false);
            assert.equal((4).isEven(), true);
            assert.equal((2).isEven(), true);
        }
    }

});

